@@ -2,6 +2,7 @@
 #include "config.h"
 #endif
 #include <string.h>
+#include <ctype.h> /* for isspace() */
 #include <gmp.h>
 #include <mpfr.h>
 #include "uint32_max.h"
@@ -17,33 +18,45 @@
 #include "memwatch.h"
 #endif
 
-void explain_command(char *);
-void explain_variable(char *);
-void explain_constant(char *);
-void explain_function(char *);
+static void explain_command(const char *);
+static void explain_variable(const char *);
+static void explain_constant(const char *);
+static void explain_function(const char *);
 
-void explain(char *str)
+void explain(const char *str)
 {				       /*{{{ */
+    int curs;
+    char * mystr;
+
     if (!str || !*str) {
+	printf("Nothing to explain.\n");
 	return;
     }
-    if (!strcmp(str, "q")) {
+    mystr = strdup(str);
+    /* for sanity's sake, remove any trailing whitespace */
+    curs = strlen(mystr) - 1;
+    while (isspace(mystr[curs]) && curs > 0) {
+	mystr[curs] = 0;
+	curs--;
+    }
+    if (!strcmp(mystr, "q")) {
 	printf("Quits the program.");
-    } else if (*str == '\\') {	       // it's a command
-	explain_command(str);
-    } else if (isconst(str)) {	       // it's a constant
-	explain_constant(str);
-    } else if (isfunc(str)) {	       // it's a function
-	explain_function(str);
-    } else if (varexists(str)) {       // it's a variable
-	explain_variable(str);
+    } else if (*mystr == '\\') {	       // it's a command
+	explain_command(mystr);
+    } else if (isconst(mystr)) {	       // it's a constant
+	explain_constant(mystr);
+    } else if (isfunc(mystr)) {	       // it's a function
+	explain_function(mystr);
+    } else if (varexists(mystr)) {       // it's a variable
+	explain_variable(mystr);
     } else {			       // it's a call for help
 	printf("%s is neither a command, constant, function, or variable.\n",
-	       str);
+	       mystr);
     }
+    free(mystr);
 }				       /*}}} */
 
-void explain_command(char *str)
+static void explain_command(const char *str)
 {				       /*{{{ */
     str++;
     if (!strcmp(str, "b") || !strcmp(str, "bin") || !strcmp(str, "binary")) {
@@ -127,13 +140,14 @@ void explain_command(char *str)
 	printf
 	    ("Gives you information about commands, variables, constants and functions.\n");
     } else if (!strcmp(str, "cmod")) {
-	printf("Changes how the modulus operator (%%) behaves with negative numbers.\n The default is to behave like the C programming language modulus, the other is slightly more flexible. For example, with the default setting:\n\n\t-340 %% 60 == -40; 340 %% -60 == 40; -340 %% -60 == -40\n\nWhen this setting is toggled, it behaves like this:\n\n\t-340 %% 60 == -40; 340 %% -60 == -20; -340 %% -60 == 20\n");
+	printf
+	    ("Changes how the modulus operator (%%) behaves with negative numbers.\n The default is to behave like the C programming language modulus, the other is slightly more flexible. For example, with the default setting:\n\n\t-340 %% 60 == -40; 340 %% -60 == 40; -340 %% -60 == -40\n\nWhen this setting is toggled, it behaves like this:\n\n\t-340 %% 60 == -40; 340 %% -60 == -20; -340 %% -60 == 20\n");
     } else {
 	printf("Undefined command.\n");
     }
 }				       /*}}} */
 
-void explain_variable(char *str)
+static void explain_variable(const char *str)
 {				       /*{{{ */
     struct answer var;
 
@@ -149,13 +163,13 @@ void explain_variable(char *str)
 	printf("%s uses the following variables:\n", str);
 	strings = extract_vars(var.exp);
 	while (listLen(strings) > 0) {
-	    char *str = (char *)getHeadOfList(strings);
-	    char *value = evalvar(str);
+	    char *curstr = (char *)getHeadOfList(strings);
+	    char *value = evalvar(curstr);
 
-	    printf("\t%s\t(currently: %s)\n", str,
+	    printf("\t%s\t(currently: %s)\n", curstr,
 		   value ? value : "undefined");
-	    if (str) {
-		free(str);
+	    if (curstr) {
+		free(curstr);
 	    }
 	    if (value) {
 		free(value);
@@ -176,7 +190,7 @@ void explain_variable(char *str)
     }
 }				       /*}}} */
 
-void explain_constant(char *str)
+static void explain_constant(const char *str)
 {				       /*{{{ */
     if (!strcmp(str, "e")) {
 	printf
@@ -236,7 +250,7 @@ void explain_constant(char *str)
     } else if (!strcmp(str, "b")) {
 	printf
 	    ("Wein displacement constant. Defines the relationship between the thermodynamic temperature of a black body and the wavelength at which the intensity of the radiation is the greatest.");
-    } else if (!strcmp(str, "ao")) {
+    } else if (!strcmp(str, "a0")) {
 	printf
 	    ("Bohr radius. A single electron orbiting at the lowest possible energy in a hydrogen atom orbits at this distance.");
     } else if (!strcmp(str, "F")) {
@@ -298,14 +312,16 @@ void explain_constant(char *str)
 	printf
 	    ("Magnetic flux quantum. The quantum of magnetic flux passing through a superconductor.");
     } else if (!strcmp(str, "K")) {
-	printf("Catalan's constant commonly appears in estimates of combinatorial functions and in certain classes of sums and definite integrals.");
+	printf
+	    ("Catalan's constant commonly appears in estimates of combinatorial functions and in certain classes of sums and definite integrals.");
     } else if (!strcmp(str, "sinc")) {
-	printf("Sinus cardinalis, also known as the interpolation function, filtering function, or the first spherical Bessel function, is the product of a sine function and a monotonically decreasing function.");
+	printf
+	    ("Sinus cardinalis, also known as the interpolation function, filtering function, or the first spherical Bessel function, is the product of a sine function and a monotonically decreasing function.");
     }
     printf("\n");
 }				       /*}}} */
 
-void explain_function(char *str)
+static void explain_function(const char *str)
 {				       /*{{{ */
     if (!strcmp(str, "sin") || !strcmp(str, "cos") || !strcmp(str, "tan") ||
 	!strcmp(str, "cot")) {
@@ -355,11 +371,14 @@ void explain_function(char *str)
     } else if (!strcmp(str, "comp")) {
 	printf("Returns the one's complement of the input number.");
     } else if (!strcmp(str, "zeta")) {
-	printf("Returns the Riemann zeta function of the input number. This is used primarily in number theory, but also has applications in physics, probability theory, and applied statistics.");
+	printf
+	    ("Returns the Riemann zeta function of the input number. This is used primarily in number theory, but also has applications in physics, probability theory, and applied statistics.");
     } else if (!strcmp(str, "Gamma")) {
-	printf("Returns the Gamma function of the input number. The Gamma function extends the factorial function to complex and non-natural numbers.");
+	printf
+	    ("Returns the Gamma function of the input number. The Gamma function extends the factorial function to complex and non-natural numbers.");
     } else if (!strcmp(str, "lnGamma")) {
-	printf("Returns the natural log of the Gamma function of the input number.");
+	printf
+	    ("Returns the natural log of the Gamma function of the input number.");
     }
     printf("\n");
 }				       /*}}} */
