@@ -3,18 +3,18 @@
 #include "memwatch.h"
 #endif
 
-static char *consts[] = {
+const char *consts[] = {
     "e","pi","PI","pI","Pi","random","irandom","Na","NA",
     "k","Cc","ec","R","G","g","Me","Mp","Mn","Md","u",
     "amu","c","h","mu0","muzero","muZERO","epsilonzero",
     "epsilonZERO","epsilon0","EPSILONzero","EPSILONZERO","EPSILON0",
-    "muB","b","ao","F","Vm","NAk","eV","sigma","alpha","gamma","re",
+    "muB","b","a0","F","Vm","NAk","eV","sigma","alpha","gamma","re",
     "Kj","Rk","Rinf","Eh","Gf","Mmu","Mt","Mtau","Mh","Malpha","n0",
     "nzero","c1","c2","G0","Gzero","Z0","Zzero","Phi0","Phizero","PhiZERO",
     0
 };
 
-int isconst(char *str)
+int isconst(const char *str)
 {
     size_t i;
 
