@@ -1,6 +1,6 @@
 #ifndef WCALC_ISCONST
 #define WCALC_ISCONST
 
-int isconst(char *str);
+int isconst(const char *str);
 
 #endif
