@@ -1,9 +1,10 @@
 #include <string.h>
+#include "config.h"
 #ifdef MEMWATCH
 #include "memwatch.h"
 #endif
 
-static char *funcs[] = { "sin", "cos", "tan", "cot",
+const char *funcs[] = { "sin", "cos", "tan", "cot",
     "asin", "acos", "atan", "acot",
     "arcsin", "arccos", "arctan", "arccot",
     "sinh", "cosh", "tanh", "coth",
@@ -13,12 +14,15 @@ static char *funcs[] = { "sin", "cos", "tan", "cot",
     "round", "abs", "floor", "ceil", "ceiling",
     "sqrt", "cbrt",
     "exp", "fact", "comp",
+#ifdef HAVE_MPFR_22
+    "eint","Gamma","gamma","lngamma","lnGamma",
+#endif
     "rand", "irand",
-    "Gamma","lngamma","lnGamma","zeta","sinc",
+    "zeta","sinc",
     0
 };
 
-int isfunc(char *str)
+int isfunc(const char *str)
 {
     size_t i;
 
