@@ -14,8 +14,7 @@
 #include <errno.h>
 #include <fcntl.h>		       /* for open() */
 #include <limits.h>		       /* for stroul() and PATH_MAX */
-#include <gmp.h>
-#include <mpfr.h>		       /* for mpfr_t */
+#include "number.h"
 
 #ifdef HAVE_LIBREADLINE
 # if defined(HAVE_READLINE_READLINE_H)
@@ -57,6 +56,7 @@ extern int history_truncate_file(char *, int);
 #endif
 
 #define TRUEFALSE (! strcmp(value,"yes") || ! strcmp(value,"true") || ! strcmp(value,"1"))
+#define BIG_STRING 4096
 
 static int read_prefs(char *filename);
 static int read_preload(char *filename);
@@ -70,15 +70,203 @@ static int read_preload(char *filename);
 extern int yyparse(void);
 extern int yy_scan_string(const char *);
 
+#ifdef HAVE_LIBREADLINE
+static List tc_options = NULL;
+
+char *tc_generator(const char *text, int state)
+{				       /*{{{ */
+    char *ret = getHeadOfList(tc_options);
+
+    if (ret) {
+	return ret;
+    } else
+	return NULL;
+}				       /*}}} */
+
+char *tc_rounding(const char *text, int state)
+{				       /*{{{ */
+    static unsigned int i = 0;
+    char *rounding[] = { "none", "simple", "sig_fig", 0 };
+    if (state == 0) {
+	i = 0;
+    }
+    while (rounding[i] != 0) {
+	if (strncmp(text, rounding[i], strlen(text)) == 0) {
+	    return strdup(rounding[i++]);
+	}
+	i++;
+    }
+    return NULL;
+}				       /*}}} */
+
+#define COMPLETE(strs) do { \
+    int compareword = 0; \
+    int comparechar = 0; \
+    int textcurs = 0; \
+    while (strs[compareword] != NULL) { \
+	if (text[textcurs] == 0) { /* add to the list of possibilities */ \
+	    addToList(&tc_options, strdup(strs[compareword])); \
+	    textcurs = 0; \
+	    comparechar = 0; \
+	    compareword++; \
+	} else if (text[textcurs] == strs[compareword][comparechar]) { \
+	    textcurs++; \
+	    comparechar++; \
+	} else { /* not a possibility: next! */ \
+	    textcurs = 0; \
+	    compareword++; \
+	    comparechar = 0; \
+	} \
+    } \
+} while (0)
+
+char **wcalc_completion(const char *text, int start, int end)
+{				       /*{{{ */
+    /*extern const char *commands[];*/
+    extern const char *qcommands[];
+    extern const char *consts[];
+    extern const char *funcs[];
+    char **variables;
+    char **retvals = NULL;
+
+    //printf("\ncompleting: %s\n", text);
+    if ('\\' == rl_line_buffer[0]) {
+	if (NULL == strchr(rl_line_buffer, ' ') &&
+	    NULL == strchr(rl_line_buffer, '\t')) {
+	    COMPLETE(qcommands);
+	} else if (strncmp("\\explain", rl_line_buffer, 8) == 0) {
+	    int i = 8;
+
+	    while (isspace(rl_line_buffer[i]))
+		++i;
+	    if (i == start) {
+		COMPLETE(qcommands);
+		COMPLETE(consts);
+		COMPLETE(funcs);
+		variables = listvarnames();
+		COMPLETE(variables);
+		free(variables);
+	    }
+	} else if (strncmp("\\open", rl_line_buffer, 5) == 0 ||
+		   strncmp("\\save", rl_line_buffer, 5) == 0) {
+	    int i = 5;
+
+	    while (isspace(rl_line_buffer[i]))
+		++i;
+	    if (i == start) {
+		retvals =
+		    rl_completion_matches(text,
+					  rl_filename_completion_function);
+		return retvals;
+	    }
+	} else
+	    if ((strncmp("\\rou", rl_line_buffer, 4) == 0 &&
+		 isspace(rl_line_buffer[4])) ||
+		(strncmp("\\round", rl_line_buffer, 6) == 0 &&
+		 isspace(rl_line_buffer[6])) ||
+		(strncmp("\\rounding", rl_line_buffer, 9) == 0 &&
+		 isspace(rl_line_buffer[9]))) {
+	    int i = 4;
+
+	    while (!isspace(rl_line_buffer[i]))
+		++i;
+	    while (isspace(rl_line_buffer[i]))
+		++i;
+	    if (i == start) {
+		retvals = rl_completion_matches(text, tc_rounding);
+		return retvals;
+	    }
+	} else
+	    if ((strncmp("\\c", rl_line_buffer, 2) == 0 &&
+		 isspace(rl_line_buffer[2])) ||
+		(strncmp("\\conv", rl_line_buffer, 5) == 0 &&
+		 isspace(rl_line_buffer[5])) ||
+		(strncmp("\\convert", rl_line_buffer, 8) == 0 &&
+		 isspace(rl_line_buffer[8]))) {
+	    int i = 2;
+	    /*extern const struct conversion lengths[], areas[], volumes[],
+		masses[], speeds[], powers[], forces[], accelerations[];*/
+	    extern const struct conversion *conversions[];
+
+	    while (!isspace(rl_line_buffer[i]))
+		++i;
+	    while (isspace(rl_line_buffer[i]))
+		++i;
+	    if (i == start) {
+		/* complete on ALL units */
+		size_t unit, conversion;
+
+		for (conversion = 0; conversions[conversion]; conversion++) {
+		    for (unit = 0; conversions[conversion][unit].name; unit++) {
+			COMPLETE(conversions[conversion][unit].aka);
+		    }
+		}
+	    } else {
+		/* seek past the previous unit... */
+		char *unit1 = rl_line_buffer + i;
+		char saved_char;
+		int unit_cat;
+
+		while (!isspace(rl_line_buffer[i]))
+		    ++i;
+		saved_char = rl_line_buffer[i];
+		rl_line_buffer[i] = 0;
+		unit_cat = identify_unit(unit1);
+		rl_line_buffer[i] = saved_char;
+		if (unit_cat != -1) {
+		    while (isspace(rl_line_buffer[i]))
+			++i;
+		    if (i == start) {
+			size_t unit;
+
+			/* complete on COMPATABLE units */
+			for (unit = 0; conversions[unit_cat][unit].name;
+			     unit++) {
+			    COMPLETE(conversions[unit_cat][unit].aka);
+			}
+			addToList(&tc_options, strdup("to"));
+		    } else if (strncmp(rl_line_buffer + i, "to", 2) == 0 &&
+			       isspace(rl_line_buffer[i + 2])) {
+			i += 2;
+			while (isspace(rl_line_buffer[i]))
+			    ++i;
+			if (i == start) {
+			    size_t unit;
+
+			    /* complete on COMPATABLE units */
+			    for (unit = 0; conversions[unit_cat][unit].name;
+				 unit++) {
+				COMPLETE(conversions[unit_cat][unit].aka);
+			    }
+			}
+		    }
+		}
+	    }
+	}
+    } else {
+	COMPLETE(qcommands);
+	COMPLETE(consts);
+	COMPLETE(funcs);
+	variables = listvarnames();
+	COMPLETE(variables);
+	free(variables);
+    }
+    rl_attempted_completion_over = 1;  // do not use standard file completion
+    rl_completion_entry_function = tc_generator;
+    retvals = rl_completion_matches(text, tc_generator);
+    return retvals;
+}				       /*}}} */
+#endif
+
 int main(int argc, char *argv[])
-{
+{				       /*{{{ */
     extern int yydebug;
     extern int lines;
 
 #ifdef HAVE_LIBREADLINE
     char *readme = NULL;
 #else
-    char readme[1000];
+    char readme[BIG_STRING];
 #endif
     int tty, i;
     short cmdline_input = 0;
@@ -104,20 +292,31 @@ int main(int argc, char *argv[])
     conf.verbose = 0;
     conf.c_style_mod = 1;
 
-    mpfr_set_default_prec(1024);
-    mpfr_init_set_ui(last_answer, 0, GMP_RNDN);
+    init_numbers();
+    num_init_set_ui(last_answer, 0);
 
     /* load the preferences */
     {
 	char filename[PATH_MAX];
+	int foundflag = 0;
 
-	snprintf(filename, PATH_MAX, "%s/.wcalcrc", getenv("HOME"));
-	if (read_prefs(filename)) {
-	    perror("Writing Preferences");
+	/* quick check the commandline for a --defaults argument */
+	for (i = 1; i < argc; ++i) {
+	    if (!strcmp(argv[i], "--defaults")) {
+		foundflag = 1;
+		break;
+	    }
 	}
-	snprintf(filename, PATH_MAX, "%s/.wcalc_preload", getenv("HOME"));
-	if (read_preload(filename)) {
-	    perror("Reading Preload File");
+
+	if (foundflag == 0) {
+	    snprintf(filename, PATH_MAX, "%s/.wcalcrc", getenv("HOME"));
+	    if (read_prefs(filename)) {
+		perror("Writing Preferences");
+	    }
+	    snprintf(filename, PATH_MAX, "%s/.wcalc_preload", getenv("HOME"));
+	    if (read_preload(filename)) {
+		perror("Reading Preload File");
+	    }
 	}
     }
 
@@ -132,7 +331,7 @@ int main(int argc, char *argv[])
 		fprintf(stderr,
 			"-P option requires a valid integer without spaces.\n");
 		fflush(stderr);
-		mpfr_clear(last_answer);
+		num_free(last_answer);
 		exit(EXIT_FAILURE);
 	    } else {
 		conf.precision = (int)argnum;
@@ -142,23 +341,23 @@ int main(int argc, char *argv[])
 	    conf.engineering = !conf.engineering;
 	} else if (!strcmp(argv[i], "-H") || !strcmp(argv[i], "--help")) {
 	    print_command_help();
-	    mpfr_clear(last_answer);
+	    num_free(last_answer);
 	    exit(EXIT_SUCCESS);
 	} else if (!strcmp(argv[i], "-v") || !strcmp(argv[i], "--version")) {
 	    printf("wcalc %s\n", VERSION);
-	    mpfr_clear(last_answer);
+	    num_free(last_answer);
 	    exit(EXIT_SUCCESS);
 	} else if (!strcmp(argv[i], "-d") || !strcmp(argv[i], "--decimal") ||
-		   !strcmp(argv[i], "-dec")) {
+		   !strcmp(argv[i], "-dec") || !strcmp(argv[i], "--dec")) {
 	    conf.output_format = DECIMAL_FORMAT;
 	} else if (!strcmp(argv[i], "-h") || !strcmp(argv[i], "--hexadecimal")
-		   || !strcmp(argv[i], "-hex")) {
+		   || !strcmp(argv[i], "-hex") || !strcmp(argv[i], "--hex")) {
 	    conf.output_format = HEXADECIMAL_FORMAT;
 	} else if (!strcmp(argv[i], "-o") || !strcmp(argv[i], "--octal") ||
-		   !strcmp(argv[i], "-oct")) {
+		   !strcmp(argv[i], "-oct") || !strcmp(argv[i], "--oct")) {
 	    conf.output_format = OCTAL_FORMAT;
 	} else if (!strcmp(argv[i], "-b") || !strcmp(argv[i], "--binary") ||
-		   !strcmp(argv[i], "-bin")) {
+		   !strcmp(argv[i], "-bin") || !strcmp(argv[i], "--bin")) {
 	    conf.output_format = BINARY_FORMAT;
 	} else if (!strcmp(argv[i], "-p") || !strcmp(argv[i], "--prefixes")) {
 	    conf.print_prefixes = !conf.print_prefixes;
@@ -220,10 +419,16 @@ int main(int argc, char *argv[])
 	    if (endptr != NULL && (strlen(endptr) > 0)) {
 		fprintf(stderr,
 			"--bits option requires a valid integer without spaces.\n");
-		mpfr_clear(last_answer);
+		num_free(last_answer);
 		exit(EXIT_FAILURE);
 	    } else {
-		mpfr_set_default_prec(argnum);
+		if (argnum < NUM_PREC_MIN) {
+		    fprintf(stderr,"Minimum precision is %lu.\n", (unsigned long)NUM_PREC_MIN);
+		} else if (argnum > NUM_PREC_MAX) {
+		    fprintf(stderr,"Maximum precision is %lu.\n", (unsigned long)NUM_PREC_MAX);
+		} else {
+		    num_set_default_prec(argnum);
+		}
 	    }
 	} else if (!strcmp(argv[i], "--ints")) {
 	    conf.print_ints = !conf.print_ints;
@@ -235,6 +440,8 @@ int main(int argc, char *argv[])
 	    yydebug = 1;
 	} else if (!strcmp(argv[i], "-n")) {
 	    conf.print_equal = 0;
+	} else if (!strcmp(argv[i], "--defaults")) {
+	    /* ignore this argument */
 	} else {
 	    extern char *errstring;
 
@@ -288,8 +495,7 @@ int main(int argc, char *argv[])
 #endif
 	clearHistory();
 	cleanupvar();
-	mpfr_clear(last_answer);
-	mpfr_free_cache();
+	num_free(last_answer);
 	lists_cleanup();
 	exit(EXIT_SUCCESS);
     }
@@ -307,6 +513,10 @@ int main(int argc, char *argv[])
 		perror("Reading History");
 	    }
 	}
+#endif
+#ifdef HAVE_LIBREADLINE
+	rl_attempted_completion_function = wcalc_completion;
+	rl_basic_word_break_characters = " \t\n\"\'+-*/[{()}]=<>!|~&^%";
 #endif
 	printf
 	    ("Enter an expression to evaluate, q to quit, or ? for help:\n");
@@ -320,11 +530,11 @@ int main(int argc, char *argv[])
 		char c;
 		unsigned int i = 0;
 
-		memset(readme, 0, 1000);
+		memset(readme, 0, BIG_STRING);
 		printf("-> ");
 		fflush(stdout);
 		c = fgetc(stdin);
-		while (c != '\n' && i < 1000 && !feof(stdin) &&
+		while (c != '\n' && i < BIG_STRING && !feof(stdin) &&
 		       !ferror(stdin)) {
 		    readme[i] = c;
 		    c = fgetc(stdin);
@@ -379,11 +589,8 @@ int main(int argc, char *argv[])
 		    extern char *errstring;
 
 		    if (errstring && strlen(errstring)) {
-			fprintf(stderr, "%s", errstring);
-			if (errstring[strlen(errstring) - 1] != '\n')
-			    fprintf(stderr, "\n");
-			free(errstring);
-			errstring = NULL;
+			display_and_clear_errstring();
+			/*rl_stuff_char('f');*/
 		    }
 		}
 	    }
@@ -404,8 +611,10 @@ int main(int argc, char *argv[])
     } else {
 	/* if stdin is ANYTHING ELSE (a pipe, a file, etc), don't prompt */
 	char *line, gotten;
-	unsigned int linelen = 0, maxlinelen = 100;
+	unsigned int linelen = 0, maxlinelen = BIG_STRING;
+	extern int show_line_numbers;
 
+	show_line_numbers = 1;
 	while (1) {
 	    line = calloc(maxlinelen, sizeof(char));
 	    gotten = fgetc(stdin);
@@ -415,15 +624,15 @@ int main(int argc, char *argv[])
 		linelen++;
 		if (linelen > maxlinelen) {
 		    char *temp;
-		    temp = realloc(line, (maxlinelen + 100) * sizeof(char));
+		    temp = realloc(line, (maxlinelen + BIG_STRING) * sizeof(char));
 		    if (!temp) {
 			free(line);
 			fprintf(stderr,
 				"Ran out of memory. Line too long.\n");
 			exit(EXIT_FAILURE);
 		    }
-		    memset(temp + maxlinelen, 0, 100);
-		    maxlinelen += 100;
+		    memset(temp + maxlinelen, 0, BIG_STRING);
+		    maxlinelen += BIG_STRING;
 		    line = temp;
 		}
 	    }
@@ -439,14 +648,16 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "error storing last answer\n");
 	    }
 	    {
-		extern char *errstring;
+		extern int errloc;
 
-		if (errstring && strlen(errstring)) {
-		    fprintf(stderr, "%s", errstring);
+		if (errloc != -1) {
+		    errloc = -1;
+		    display_and_clear_errstring();
+		    /*fprintf(stderr, "%s", errstring);
 		    if (errstring[strlen(errstring) - 1] != '\n')
 			fprintf(stderr, "\n");
 		    free(errstring);
-		    errstring = NULL;
+		    errstring = NULL;*/
 		}
 	    }
 	    free(line);
@@ -458,19 +669,19 @@ int main(int argc, char *argv[])
     cleanupvar();
     if (pretty_answer) {
 	extern char *pa;
+
 	free(pretty_answer);
 	if (pa) {
 	    free(pa);
 	}
     }
-    mpfr_clear(last_answer);
-    mpfr_free_cache();
+    num_free(last_answer);
     lists_cleanup();
     exit(EXIT_SUCCESS);
-}
+}				       /*}}} */
 
 static int read_preload(char *filename)
-{
+{				       /*{{{ */
     struct stat sb;
 
     if (-1 == stat(filename, &sb)) {
@@ -481,78 +692,83 @@ static int read_preload(char *filename)
 	}
     }
     return loadState(filename, 0);
-}
+}				       /*}}} */
 
 static int read_prefs(char *filename)
-{
+{				       /*{{{ */
     int fd = open(filename, O_RDONLY);
-    char key[1000], value[100];
-    char *curs = key;
+    char key[BIG_STRING], value[BIG_STRING];
+    size_t curs = 0;
     int retval = 1;
 
     fd = open(filename, O_RDONLY);
     if (fd < 0)
 	return 0;
-    retval = read(fd, curs, 1);
+    retval = 1;
     while (retval == 1) {
 	char quoted = 0;
+	char junk;
 
-	curs = key;
+	memset(value, 0, BIG_STRING);
+	memset(key, 0, BIG_STRING);
+
+	curs = 0;
 	// read until we find a non-comment
-	while (retval == 1) {
+	while (1 == (retval = read(fd, &(key[curs]), 1))) {
 	    // if we find a comment
-	    if (*curs == '#')
+	    if (key[curs] == '#') {
 		// read until the end of line
-		while (read(fd, curs, 1) == 1 && *curs != '\n') ;
-	    else if (isalpha((int)(*curs)))
+		junk = 0;
+		while (read(fd, &junk, 1) == 1 && junk != '\n') ;
+	    } else if (isalpha((int)(key[curs])))
 		break;
-	    retval = read(fd, curs, 1);
 	}
+	if (retval != 1)
+	    break;
 	// read in the key
 	quoted = 1;
-	if (*curs != '\'') {
-	    ++curs;
+	if (key[curs] != '\'') {
+	    curs++;
 	    quoted = 0;
 	}
-	while (retval == 1) {
-	    retval = read(fd, curs, 1);
-	    if ((!quoted && isspace((int)(*curs))) ||
-		(quoted && *curs != '\''))
+	while (retval == 1 && curs < BIG_STRING) {
+	    retval = read(fd, &(key[curs]), 1);
+	    if ((!quoted && isspace((int)(key[curs]))) ||
+		(quoted && key[curs] != '\''))
 		break;
 	    ++curs;
 	}
 	if (retval != 1)
 	    break;
-	*curs = 0;
+	key[curs] = 0;
 	// read in the =
-	while (retval == 1 && *curs != '=') {
-	    retval = read(fd, curs, 1);
+	junk = 0;
+	while (retval == 1 && junk != '=') {
+	    retval = read(fd, &junk, 1);
 	}
 	if (retval != 1)
 	    break;
-	while (retval == 1 && (isspace((int)(*curs)) || *curs == '=')) {
-	    retval = read(fd, curs, 1);
+	while (retval == 1 && (isspace((int)junk) || junk == '=')) {
+	    retval = read(fd, &junk, 1);
 	}
 	if (retval != 1)
 	    break;
-	value[0] = *curs;
-	*curs = 0;
-	curs = value;
+	value[0] = junk;
+	curs = 0;
 	// read in the value
 	quoted = 1;
-	if (*curs != '\'') {
+	if (value[0] != '\'') {
 	    ++curs;
 	    quoted = 0;
 	}
-	while (retval == 1) {
-	    retval = read(fd, curs, 1);
-	    if ((!quoted && isspace((int)(*curs))) ||
-		(quoted && *curs != '\''))
+	while (retval == 1 && value[curs-1] && curs < BIG_STRING) {
+	    retval = read(fd, &(value[curs]), 1);
+	    if ((!quoted && isspace((int)(value[curs]))) ||
+		(quoted && value[curs] != '\''))
 		break;
-	    ++curs;
+	    curs++;
 	}
-	if (*curs == '\n')
-	    *curs = 0;
+	value[curs] = 0;
 
 	if (!strcmp(key, "precision"))
 	    conf.precision = atoi(value);
@@ -568,6 +784,8 @@ static int read_prefs(char *filename)
 	    conf.print_prefixes = TRUEFALSE;
 	else if (!strcmp(key, "save_errors"))
 	    conf.remember_errors = TRUEFALSE;
+	else if (!strcmp(key, "remember_errors"))
+	    conf.remember_errors = TRUEFALSE;
 	else if (!strcmp(key, "precision_guard"))
 	    conf.precision_guard = TRUEFALSE;
 	else if (!strcmp(key, "print_integers"))
@@ -610,4 +828,4 @@ static int read_prefs(char *filename)
 	memset(value, 0, sizeof(value));
     }
     return 0;
-}
+}				       /*}}} */
